#!/bin/bash
# ================================================================================
#
# TRANSFER :: PLUGIN - DUPLICITY
#
# this script will be included in ../../transfer.sh
#
# --------------------------------------------------------------------------------
# ah - Axel Hahn <axel.hahn@iml.unibe.ch>
# 2021-05-19  ah    v0.0   INIT ... WIP
# ================================================================================

# --------------------------------------------------------------------------------
# ENV VARS
# export variables that are needed by the backup tool
# --------------------------------------------------------------------------------

export PASSPHRASE

# --------------------------------------------------------------------------------
# INIT
# --------------------------------------------------------------------------------

    # check requirements
    function t_checkRequirements(){
        echo "DEBUG: function t_checkRequirements"

        j_requireUser "root"
        j_requireBinary "duplicity"
    }

    # set variables
    function t_setVars(){
        echo "DEBUG: t_setVars"

    }

# --------------------------------------------------------------------------------
# GENERATE PARAMS :: ALL DIRS
# --------------------------------------------------------------------------------

    # return a string with default params
    # param  string  param1 of transfer.sh; one of full|inc|auto
    # param  string  param2 of transfer.sh; for auto: date i.e. 3M for 3 monthes
    function t_getParamDefault(){

        local _method=
        local _volsize=

        # --- method
        test "$1" = "full" && _method="full"
        test "$1" = "auto" && _method="--full-if-older-than $2"
        echo -n "$_method"

        # --- backend
        # task#1623 - fallback ssh backend for Debian 8
        sSshBackend=`_j_getvar ${STORAGEFILE} "ssh-backend"`
        if [ ! -z $sSshBackend ]; then
            echo -n " --ssh-backend $sSshBackend"
        fi

        # --- volume size
        _volsize=`_j_getvar ${STORAGEFILE} "duplicity_volsize"`
        test -z "$_volsize" && _volsize=`_j_getvar ${STORAGEFILE} "volsize"`

        if [ ! -z $_volsize ]; then
            echo -n " --volsize ${_volsize}"
        fi

        # --- verbosity level to fetch changed files from log
        echo -n " -v8"

        # --- add asynchronous upload
        echo -n " --asynchronous-upload"

    }
    # return a string with backup default params
    function t_getParamBackup(){
        echo -n ''
    }
    # return a cli parameter for a single exlude directory
    # param  string  cache directory for local index files
    function t_getParamCacheDir(){
    if [ ! -z "$1" ]; then
        local sCacheDir="$1"
        if [ ! -d $sCacheDir ]; then
            mkdir -p $sCacheDir
            chmod 750 $sCacheDir
        fi
        echo --archive-dir=$sCacheDir
    fi
    }

    # return a cli parameter for a single exlude directory
    # param  string  exlude pattern
    function t_getParamExlude(){
        test -z "$1" || echo --exclude-regexp "'"$*"'"
    }
    # return a cli parameter for a single exlude directory
    # param  string  exlude pattern
    function t_getParamInlude(){
        test -z "$1" || echo --include-regexp "'"$*"'"
    }

    # return a cli parameter to use an ssh keyfile
    # param  string  filename if ssh private key file
    function t_getParamSshKey(){
        test -z "$1" || echo --ssh-options="'"-oIdentityFile=$1"'"
    }

# --------------------------------------------------------------------------------
# ACTIONS :: TRANSFER
# --------------------------------------------------------------------------------
    # pre backup actions
    # uses global vars from ../../transfer.sh
    function t_cmdPre(){
        echo No PRE actions before starting transfer
    }


    # post backup actions
    # uses global vars from ../../transfer.sh
    function t_cmdPost(){
        echo No POST actions after all transfers
    }

# --------------------------------------------------------------------------------
# GENERATE PARAMS :: SINGLE DIR
# --------------------------------------------------------------------------------

    # get target url/ directory
    # param  string  directory to backup
    function t_sd_getTarget(){
        # duplicity has a directory based target
        j_getFullTarget "$1"
    }

    # get string with complete backup command
    function t_sd_getCmdBackup(){
        echo duplicity ${sBackupParams} ${mydir} ${sTarget}
    }

    # pre backup tasks
    # uses global vars from ../../transfer.sh
    function t_sd_cmdPre(){
        # --- for rsync only: create remote directory
        echo ${sTarget} | fgrep "rsync://"  >/dev/null
        if [ $? -eq 0 ]; then
          # sshTarget=`echo ${sTarget} | sed "s#rsync://#scp://#"`
          # echo Creating remote directory with fetching collection-status on $sshTarget
          # color cmd
          # duplicity collection-status ${sParams} ${sshTarget}
          # color reset
          sshTarget=`echo ${sTarget} | cut -f 3 -d '/'`
          RemoteDir=`echo ${sTarget} | cut -f 4- -d '/'`
          cmd="ssh"
          if [ ! -z ${sFileSshPrivkey} ]; then
            cmd="${cmd} -i ${sFileSshPrivkey}"
          fi
          cmd="${cmd} ${sshTarget} mkdir -p ${RemoteDir} 2>/dev/null ; ls -ld ${RemoteDir} "
          echo Creating remote directory first ...
          color cmd
          $cmd
          color reset    
        fi
    }

    # post backup tasks
    # uses global vars from ../../transfer.sh
    function t_sd_cmdPost(){
        local STORAGE_KEEP=`_j_getvar ${STORAGEFILE} "duplicity_keep"`
        test -z "$STORAGE_KEEP"    && STORAGE_KEEP=`_j_getvar ${STORAGEFILE} "keep"`

        if [ -z "$STORAGE_KEEP" ]; then
            color error
            echo ERROR: missing config for backup target.
            echo There must be an entry keep in ${STORAGEFILE}
            color reset
            exit 1
        fi

        cmd="duplicity remove-older-than $STORAGE_KEEP --force ${sParams} ${sTarget}"
        echo $cmd
        color cmd
        $cmd
        fetchrc
        color reset
        
        t_rcCheckCleanup $myrc

    }
# --------------------------------------------------------------------------------
# VERIFY RETURNCODES
# --------------------------------------------------------------------------------

    # init repository
    function t_rcCheckInit(){
        case $1 in
            0) color ok;      echo "OK - the repository was created." ;;
            *) color error;   echo "Verify output above - returncode of init was $1" ;;
        esac
        color reset
    }
    # backup files
    function t_rcCheckBackup(){
        case $1 in
            0) color ok;      echo "OK" ;;
            23) color error
                echo A lock file was found. Maybe this server was rebooted while performing a backup.
                echo If so delete the file lockfile.lock named in the output and start $0 again.
                ;;
            31) color error
                echo Maybe you it is a problem with the gpg-agent.conf
                ls -l ~/.gnupg/gpg-agent.conf && cat ~/.gnupg/gpg-agent.conf
                ;;
            *) color error;   echo "Backup error - returncode was $1" ;;
        esac
        color reset
    }

    # repoitory cleanup
    function t_rcCheckCleanup(){
        case $1 in
            0) color ok;      echo "OK" ;;
            *) color error;   echo "Cleanup error - returncode was $1" ;;
        esac
        color reset
    }

    # restore files
    function t_rcCheckRestore(){
        case $1 in
            0) color ok;      echo "OK" ;;
            *) color error;   echo "Restore error - returncode was $1" ;;
        esac
        color reset
    }

# --------------------------------------------------------------------------------
